addpath '../regularization' %% is Hansen's free regularization package
                            
clear all
close all
%%% parameterization of the problem:
nrec = 10;
nsrc = 10;
mx = 13;
my = 13;
n = nrec*nsrc; % number of rays
m = mx*my;     % number of cells = no. of model parameters
maxsval = min(n,m); % maximum rank of the jacobian
model = zeros(m,maxsval); % to store models as  fn. of k.

sigmatrue = 0.52;

%%% loading the traveltime pertubations:
load tttrue100x169
rawtt = tttrue100x169;
%%% loading the jacobian: %%%%%%%%%%%%%%%%
load jacobian100x169
jacobian = jacobian100x169;
rawA =partition(jacobian,m);

[times,A] =sortbyhand(rawtt,rawA);

Lm = makeoldL(mx);
Ln = get_l(n,2);

%%% computing the SVD: %%%%%%%%%%%%%%%%
[U,s,V] = csvd(A);

%%% sigma estimation by l-curve method
[reg_corner,rho,eta,reg_param] = l_curve(U,s,times);
tikhoslow = tikhonov(U,s,V,times,reg_corner);
residual = times - A*tikhoslow;
sigma_I = std(residual);

lambda = 1:400;
lambda = lambda.^2;
maxl = length(lambda);

for j = 1:maxl
  %% L-curve, with Lm on model:
  invA = adagger(A,lambda(j),Lm);
  Adagger = invA*A';
  x  =  Adagger*times; 
  Lx(j) = norm(Lm*x);
  AxminD(j)= norm(A*x-times);
 
  %% non-parametrically, with Ln on (sorted) data:
  muoperator = adagger(eye(n),lambda(j),Ln);
  mu = muoperator*times;
  Lmu(j) = norm(Ln*mu);
  muminD(j) = norm(mu-times);
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% max curvature on the L-curve:
%lambda_L = maxcurv(log(AxminD),log(Lx),lambda);
[curv_L,jmax_L] =maxcurv(log(AxminD),log(Lx),lambda);
sigma_L = AxminD(jmax_L)/sqrt(n);
lambda_L = lambda(jmax_L); 
  
[curv_mu,jmax_mu] =maxcurv(log(muminD), ...
			   log(Lmu),lambda);
sigma_mu = muminD(jmax_mu)/sqrt(n);
lambda_mu = lambda(jmax_mu);

%% L-aplace method: 
invA_L = adagger(A,lambda_L,Lm);
Adagger_L =invA_L*A';
x_L = (Adagger_L*times)';

%% max-chi-curve method:
%% chi-curv, model and sigma:
[curv_curv,jmax_curv] =maxcurv(log(AxminD),log(1./lambda),lambda);
sigma_curv = AxminD(jmax_curv)/sqrt(n);
lambda_curv = lambda(jmax_curv);

%lambda_curv = maxcurv(log(AxminD),log(1./lambda),lambda);
invA_curv = adagger(A,lambda_curv,Lm);
Adagger_curv = invA_curv*A';
x_curv = (Adagger_curv*times)'; 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

plotL(maxl,lambda,m,AxminD,Lx,jmax_L,'||Lx||','||Ax-d||');
plotL(maxl,lambda,m,AxminD,1./lambda,jmax_curv,'1/\lambda','||Ax-d||');
plotL(maxl,lambda,m,muminD,Lmu,jmax_mu,'||L\mu||','||\mu-d||');
% $$$ figure
% $$$ loglog(AxminD,1./lambda)
% $$$ for k = 1:floor(maxl/5):maxl
% $$$   text(AxminD(k),1./lambda(k),num2str(lambda(k)));
% $$$ end
% $$$ ylabel('log(1/\lambda)')
% $$$ xlabel('log||Ax-d||')
% $$$ hold off
% $$$ 
% $$$ figure  
% $$$ loglog(AxminD,Lx,'-')
% $$$ HoldState = ishold; hold on;
% $$$ for k = 1:floor(maxl/5):maxl
% $$$   text(AxminD(k),Lx(k),num2str(lambda(k)));
% $$$ end
% $$$ xlabel('log ||Ax-d||')
% $$$ ylabel('log ||Lx||') 
% $$$ hold off
% $$$ 
% $$$ figure
% $$$ mod = contourf(partition(x_L,my),20);
% $$$ hold on
% $$$ colormap('gray')
% $$$ col = colorbar;
% $$$ hold off
% $$$ 
% $$$ figure
% $$$ mod = contourf(partition(x_curv,my),20);
% $$$ hold on
% $$$ colormap('gray')
% $$$ col = colorbar;
% $$$ hold off

