%% This codes does MC estimation of sigma but over various random data errors
%% some standards: m = number of model parameters, n = number of data

clear all;
close all;
addpath '../regularization' %% is Hansen's free regularization package

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% defining the noise distribution:
% only for normally distributed noise exactvar = exact variance!!
exactsig = 2;
corlen = 0;
dist = 'N';           % 'N' is Gaussian, 'U' is uniform

[A,slowmodel,modelA,exactslowmodel,zbottom,ztop] = geometry;
[n,m] = size(A);
[U,s,V] = csvd(A); %% is the compact svd, projecting out the null space
	
%%% define the discrete Laplacian for our parametric and non-parametric
%%% error estimation:
Lm = get_l(m,2); %% get_l is a Hansen function
Ln = get_l(n,2);

%%% range of regularization parameters:
lambda =1:5:1000;
lambda =lambda.^2;
maxl = length(lambda);

%%% bounds on ||x|| and ||Lx||:
ub_x = 2*ones(m,1); 
lb_x = zeros(m,1); 
ub_Lx = 0.001*ones(m-2,1); 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% the loop over "numbreal" Monte Carlo simulations:
numbreal = 1;
h = clock;
seed = floor(10*h(6));    %% varying seed to get varying samples of noise
randn('state',seed);
for index = 1:numbreal
  %% making noisy travel times:
  [times,sigmatrue(index)] = ...
      maketimes(modelA,exactslowmodel,n,dist,exactsig,corlen);
  
  %%%%%%%%%%% 1. L=I-curve and sigma_I: %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  [lambda_I(index),rho,eta,reg_param] = l_curve(U,s,times);
  x_I(index,:)  = tikhonov(U,s,V,times,lambda_I(index))'; 
  
  % estimate of the data variance is computed from the residuals 
  % of the predicted data and the noisy observations:
  residual = times - A*x_I(index,:)';
  sigma_I(index) = std(residual); 
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  
  %% data fit and model smoothness for different lambda's:
  for j = 1:maxl
    invA = adagger(A,lambda(j),Lm);
    Adagger = invA*A';
    x = Adagger*times; 
    Lx(index,j) = norm(Lm*x);
    AxminD(index,j)= norm(A*x-times);
    
    %%%%%%%%% non-parametrically, a-la Luis: %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    muoperator = adagger(eye(n),lambda(j),Ln);
    mu = muoperator*times;
    Lmu(index,j) = norm(Ln*mu);
    muminD(index,j) = norm(mu-times);
  end 
  
  %% 2. noise estimate, non-paramtrically:
  [curv_mu,jmax_mu] = maxcurv(log(muminD(index,:)),log(Lmu(index,:)),lambda);
  sigma_mu(index) = muminD(index,jmax_mu)/sqrt(n);
  lambda_mu(index) = lambda(jmax_mu);
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  
  %% 3. max curvature on the L-aplace curve:
  [curv_L,jmax_L] = maxcurv(log(AxminD(index,:)),log(Lx(index,:)),lambda);
  sigma_L(index) = AxminD(index,jmax_L)/sqrt(n);
  lambda_L(index) = lambda(jmax_L); 

  %% L-aplace model: 
  invA_L = adagger(A,lambda_L(index),Lm); 
  Adagger_L =invA_L*A';
  x_L(index,:) = (Adagger_L*times)';
    
  %% 4. chi-curv sigma:
  [curv_curv,jmax_curv] = maxcurv(log(AxminD(index,:)),log(1./lambda),lambda);
  sigma_curv(index) = AxminD(index,jmax_curv)/sqrt(n);
  lambda_curv(index) = lambda(jmax_curv);
  
  %% variances_L on the model parameters:
  sigmaAdagAdagT(:,index) = sqrt(sigma_L(index)^2*...
  				 diag(Adagger_L*Adagger_L'));
  
  %% computing the exact bias_L with (discretized) exact model known:
  B = -lambda_L(index)*invA_L*Lm'*Lm;
  bias(:,index) = B*slowmodel'; 
  
  %% bias "limits" with constraints on x. 
  for j = 1:m
    [maxtmp,negmaxbiasx(index,j)] = linprog(-B(j,:),Lm,ub_Lx,[],[],lb_x, ub_x);
    [mintmp,minbiasx(index,j)] = linprog(B(j,:),Lm,ub_Lx,[],[],lb_x, ub_x);
  end
  maxbiasx(index,:) = negmaxbiasx(index,:); %% just the way linprog works....
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
plotL(maxl,lambda,m,AxminD(1,:),Lx(1,:),jmax_L,'||Lx||','||Ax-d||');
plotL(maxl,lambda,m,AxminD(1,:),1./lambda,jmax_curv,'1/\lambda','||Ax-d||');
plotL(maxl,lambda,m,muminD(1,:),Lmu(1,:),jmax_mu,'||L\mu||','||\mu-d||');

plotfigs(zbottom,slowmodel,x_L,m,sigmaAdagAdagT(:,1)',maxbiasx(1,:),...
	 minbiasx(1,:),bias(:,1));
